# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.

# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
"""tests for the ast utilities

Copyright (c) 2000-2003 LOGILAB S.A. (Paris, FRANCE).
http://www.logilab.fr/ -- mailto:contact@logilab.fr
"""
__revision__ = "$Id: test_astng.py,v 1.3 2003/09/12 07:59:01 syt Exp $"

import unittest
import sys
import parser
from imp import find_module,load_module
from os import getcwd
from os.path import join

from logilab.common.astng import astng, astng_objects

try:
    DATA_DIR = join(dirname(__file__), 'data')
except:
    DATA_DIR = join(getcwd(), 'data')
    
mp_file, mp_filename, mp_desc = find_module('module', [DATA_DIR])
test_module = load_module('module', mp_file, mp_filename, mp_desc)

ASTNG_REPR = astng.ASTNGBuilder().build_from_module(test_module)

#print ASTNG_REPR.struct_info()

class ASTNGRepresentationTest(unittest.TestCase):

    def test(self):
        self.assertEqual(len(ASTNG_REPR.children), 6)
        self.assertEqual(isinstance(ASTNG_REPR.children[0], astng_objects.StringStmt), 1)

        
    def test_module_base_props(self):
        self.assertEqual(ASTNG_REPR.line, 0)
        self.assertEqual(ASTNG_REPR.num, 0)
        self.assertEqual(ASTNG_REPR.__name__, 'module')
        self.assertEqual(ASTNG_REPR.file, join(DATA_DIR, 'module.py'))
        self.assertEqual(ASTNG_REPR.is_pure_python(), 1)
        self.assertEqual(ASTNG_REPR.is_package(), 0)
        self.assertEqual(ASTNG_REPR.docstring(), "test module for astng\n")

        
    def test_module_locals(self):
        _locals = ASTNG_REPR.locals()
        keys = _locals.keys()
        keys.sort()
        self.assertEqual(_locals is ASTNG_REPR.globals(), 1)
        self.assertEqual(len(_locals), 5)
        self.assertEqual(keys, ['MY_DICT', 'YO', 'YOUPI', '__revision__', 'global_access'])

        
    def test_function_base_props(self):
        function = ASTNG_REPR.locals()['global_access']

        self.assertEqual(function.line, 8)
        self.assertEqual(function.num, 4)
        self.assertEqual(function.__name__, 'global_access')
        self.assertEqual(function.args, ['key', 'val'])
        self.assertEqual(function.docstring(), 'function test')
        self.assertEqual(function.is_method(), 0)


    def test_function_locals(self):
        locals = ASTNG_REPR.locals()['global_access'].locals()
        keys = locals.keys()
        keys.sort()

        self.assertEqual(len(locals), 3)
        self.assertEqual(keys, ['key', 'local', 'val'])


    def test_class_base_props(self):
        klass = ASTNG_REPR.locals()['YO']

        self.assertEqual(klass.line, 13)
        self.assertEqual(klass.num, 8)
        self.assertEqual(klass.__name__, 'YO')
        self.assertEqual(klass.docstring(), 'hehe')


    def test_class_locals(self):
        klass1 = ASTNG_REPR.locals()['YO']
        klass2 = ASTNG_REPR.locals()['YOUPI']
        locals1 = klass1.locals()
        locals2 = klass2.locals()
        keys = locals2.keys()
        keys.sort()

        self.assertEqual(locals1, {})
        self.assertEqual(keys, ['__init__', 'class_attr', 'class_method',
                                'method', 'static_method'])


    def test_class_members(self):
        klass1 = ASTNG_REPR.locals()['YO']
        klass2 = ASTNG_REPR.locals()['YOUPI']

        self.assertEqual(klass1.instance_attrs, {})
        self.assertEqual(klass2.instance_attrs.keys(), ['member'])


    def test_class_inheritance(self):
        klass1 = ASTNG_REPR.locals()['YO']
        klass2 = ASTNG_REPR.locals()['YOUPI']

        self.assertEqual(klass1.bases, [])
        self.assertEqual(klass2.bases, ['YO'])


    def test_method_base_props(self):
        klass2 = ASTNG_REPR.locals()['YOUPI']
        method = klass2.locals()['method']

        self.assertEqual(method.line, 22)
        self.assertEqual(method.num, 14)
        self.assertEqual(method.__name__, 'method')
        self.assertEqual(method.args, ['self'])
        self.assertEqual(method.docstring(), 'method test')
        self.assertEqual(method.is_method(), 1)
        self.assertEqual(method.is_static_method(), 0)
        self.assertEqual(method.is_class_method(), 0)


    def test_method_locals(self):
        klass2 = ASTNG_REPR.locals()['YOUPI']
        method = klass2.locals()['method']
        locals = method.locals()
        keys = locals.keys()
        keys.sort()
        
        self.assertEqual(len(locals), 3)
        self.assertEqual(keys, ['MY_DICT', 'local', 'self'])
        
    def test_static_method_base_props(self):
        klass2 = ASTNG_REPR.locals()['YOUPI']
        method = klass2.locals()['static_method']

        self.assertEqual(method.args, [])
        self.assertEqual(method.is_method(), 1)
        self.assertEqual(method.is_static_method(), 1)
        self.assertEqual(method.is_class_method(), 0)
        
    def test_class_method_base_props(self):
        klass2 = ASTNG_REPR.locals()['YOUPI']
        method = klass2.locals()['class_method']

        self.assertEqual(method.args, ['cls'])
        self.assertEqual(method.is_method(), 1)
        self.assertEqual(method.is_static_method(), 0)
        self.assertEqual(method.is_class_method(), 1)
    
def suite():
    """return the unitest suite"""
    loader = unittest.TestLoader()
    testsuite = loader.loadTestsFromModule(sys.modules[__name__])
    return testsuite

def Run(runner=None):
    """run tests"""
    testsuite = suite()
    if runner is None:
        runner = unittest.TextTestRunner()
        # uncomment next line to write tests results in a file
        #runner.__init__(open('tests.log','w+'))    
    return runner.run(testsuite)
        
if __name__ == '__main__':
    Run()
